@@ -42,8 +42,17 @@ class AirtestCase(unittest.TestCase):
     def tearDown(self):
         if self.args.log and self.args.recording:
             for k, dev in enumerate(G.DEVICE_LIST):
+                # 录屏文件保存的命名规则：
+                # 如果未指定文件名，只传了--recording，就默认用recording_手机序列号.mp4来命名
+                # 如果指定了文件名--recording test.mp4，且超过一台手机，就命名为 手机序列号_test.mp4
+                # 否则直接用指定的文件名保存录屏，必须是mp4结尾
                 try:
-                    output = os.path.join(self.args.log, "recording_%d.mp4" % k)
+                    if isinstance(self.args.recording, six.string_types) and self.args.recording.endswith(".mp4"):
+                        basename = os.path.basename(self.args.recording)
+                        output_name = dev.serialno + "_" + basename if len(G.DEVICE_LIST) > 1 else basename
+                    else:
+                        output_name = "recording_{serialno}.mp4".format(serialno=dev.serialno)
+                    output = os.path.join(self.args.log, output_name)
                     dev.stop_recording(output)
                 except:
                     traceback.print_exc()
@@ -60,8 +69,7 @@ class AirtestCase(unittest.TestCase):
         try:
             exec(compile(code.encode("utf-8"), pyfilepath, 'exec'), self.scope)
         except Exception as err:
-            tb = traceback.format_exc()
-            log("Final Error", tb)
+            log(err, desc="Final Error", snapshot=True)
             six.reraise(*sys.exc_info())
 
     @classmethod
@@ -124,10 +132,16 @@ def setup_by_args(args):
     else:
         print("do not save log")
 
+    # set snapshot quality
+    if args.compress:
+        compress = args.compress
+    else:
+        compress = ST.SNAPSHOT_QUALITY
+
     # guess project_root to be basedir of current .air path
     project_root = os.path.dirname(args.script) if not ST.PROJECT_ROOT else None
 
-    auto_setup(dirpath, devices, args.log, project_root)
+    auto_setup(dirpath, devices, args.log, project_root, compress)
 
 
 def run_script(parsed_args, testcase_cls=AirtestCase):
