@@ -9,14 +9,14 @@ import warnings
 import subprocess
 import threading
 
-from six import PY3, text_type, binary_type
+from six import PY3, text_type, binary_type, raise_from
 from six.moves import reduce
 
 from airtest.core.android.constant import (DEFAULT_ADB_PATH, IP_PATTERN,
-                                           SDK_VERISON_NEW)
+                                           SDK_VERISON_ANDROID7)
 from airtest.core.error import (AdbError, AdbShellError, AirtestError,
                                 DeviceConnectionError)
-from airtest.utils.compat import decode_path
+from airtest.utils.compat import decode_path, raisefrom, proc_communicate_timeout, SUBPROCESS_FLAG
 from airtest.utils.logger import get_logger
 from airtest.utils.nbsp import NonBlockingStreamReader
 from airtest.utils.retry import retries
@@ -33,9 +33,11 @@ class ADB(object):
     status_offline = "offline"
     SHELL_ENCODING = "utf-8"
 
-    def __init__(self, serialno=None, adb_path=None, server_addr=None):
+    def __init__(self, serialno=None, adb_path=None, server_addr=None, display_id=None, input_event=None):
         self.serialno = serialno
         self.adb_path = adb_path or self.builtin_adb_path()
+        self.display_id = display_id
+        self.input_event = input_event
         self._set_cmd_options(server_addr)
         self.connect()
         self._sdk_version = None
@@ -148,11 +150,12 @@ class ADB(object):
             cmds,
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE
+            stderr=subprocess.PIPE,
+            creationflags=SUBPROCESS_FLAG
         )
         return proc
 
-    def cmd(self, cmds, device=True, ensure_unicode=True):
+    def cmd(self, cmds, device=True, ensure_unicode=True, timeout=None):
         """
         Run the adb command(s) in subprocess and return the standard output
 
@@ -160,6 +163,7 @@ class ADB(object):
             cmds: command(s) to be run
             device: if True, the device serial number must be specified by -s serialno argument
             ensure_unicode: encode/decode unicode of standard outputs (stdout, stderr)
+            timeout: timeout in seconds
 
         Raises:
             DeviceConnectionError: if any error occurs when connecting the device
@@ -170,7 +174,10 @@ class ADB(object):
 
         """
         proc = self.start_cmd(cmds, device)
-        stdout, stderr = proc.communicate()
+        if timeout:
+            stdout, stderr = proc_communicate_timeout(proc, timeout)
+        else:
+            stdout, stderr = proc.communicate()
 
         if ensure_unicode:
             stdout = stdout.decode(get_std_encoding(sys.stdout))
@@ -287,14 +294,10 @@ class ADB(object):
             None
 
         """
-        proc = self.start_cmd("wait-for-device")
-        timer = threading.Timer(timeout, proc.kill)
-        timer.start()
-        ret = proc.wait()
-        if ret == 0:
-            timer.cancel()
-        else:
-            raise DeviceConnectionError("device not ready")
+        try:
+            self.cmd("wait-for-device", timeout=timeout)
+        except RuntimeError as e:
+            raisefrom(DeviceConnectionError, "device not ready", e)
 
     def start_shell(self, cmds):
         """
@@ -347,7 +350,7 @@ class ADB(object):
             command output
 
         """
-        if self.sdk_version < SDK_VERISON_NEW:
+        if self.sdk_version < SDK_VERISON_ANDROID7:
             # for sdk_version < 25, adb shell do not raise error
             # https://stackoverflow.com/questions/9379400/adb-error-codes
             cmd = split_cmd(cmd) + [";", "echo", "---$?---"]
@@ -683,7 +686,10 @@ class ADB(object):
             command output (stdout)
 
         """
-        raw = self.cmd('shell screencap -p', ensure_unicode=False)
+        if self.display_id:
+            raw = self.cmd('shell screencap -d {0} -p'.format(self.display_id), ensure_unicode=False)
+        else:
+            raw = self.cmd('shell screencap -p', ensure_unicode=False)
         return raw.replace(self.line_breaker, b"\n")
 
     # PEP 3113 -- Removal of Tuple Parameter Unpacking
@@ -792,8 +798,8 @@ class ADB(object):
         Returns:
             The file size
         """
-        out = self.shell(["ls", "-l", filepath])
-        file_size = int(out.split()[3])
+        out = self.shell(["stat", "-c", "%s", filepath])
+        file_size = int(out)
         return file_size
 
     def _cleanup_forwards(self):
@@ -818,7 +824,7 @@ class ADB(object):
 
         """
         if not self._line_breaker:
-            if self.sdk_version >= SDK_VERISON_NEW:
+            if self.sdk_version >= SDK_VERISON_ANDROID7:
                 line_breaker = os.linesep
             else:
                 line_breaker = '\r' + os.linesep
@@ -1017,7 +1023,7 @@ class ADB(object):
             AirtestError: if top activity cannot be obtained
 
         Returns:
-            top activity as a tuple
+            top activity as a tuple: (package_name, activity_name, pid)
 
         """
         dat = self.shell('dumpsys activity top')
@@ -1056,7 +1062,13 @@ class ADB(object):
         screenOnRE = re.compile('mScreenOnFully=(true|false)')
         m = screenOnRE.search(self.shell('dumpsys window policy'))
         if m:
-            return (m.group(1) == 'true')
+            return m.group(1) == 'true'
+        else:
+            # MIUI11
+            screenOnRE = re.compile('screenState=(SCREEN_STATE_ON|SCREEN_STATE_OFF)')
+            m = screenOnRE.search(self.shell('dumpsys window policy'))
+            if m:
+                return m.group(1) == 'SCREEN_STATE_ON'
         raise AirtestError("Couldn't determine screen ON state")
 
     def is_locked(self):
@@ -1070,7 +1082,7 @@ class ADB(object):
             True or False whether the screen is locked or not
 
         """
-        lockScreenRE = re.compile('(?:mShowingLockscreen|isStatusBarKeyguard)=(true|false)')
+        lockScreenRE = re.compile('(?:mShowingLockscreen|isStatusBarKeyguard|showing)=(true|false)')
         m = lockScreenRE.search(self.shell('dumpsys window policy'))
         if not m:
             raise AirtestError("Couldn't determine screen lock state")
@@ -1160,7 +1172,7 @@ class ADB(object):
             output = ""
         if 'package:' not in output:
             raise AirtestError('package not found, output:[%s]' % output)
-        return output.split(":")[1].strip()
+        return output.split("package:")[1].strip()
 
     def check_app(self, package):
         """
@@ -1252,7 +1264,8 @@ class ADB(object):
 
     def get_ip_address(self):
         """
-        Perform several set of commands to obtain the IP address
+        Perform several set of commands to obtain the IP address.
+
             * `adb shell netcfg | grep wlan0`
             * `adb shell ifconfig`
             * `adb getprop dhcp.wlan0.ipaddress`
@@ -1263,6 +1276,26 @@ class ADB(object):
         """
 
         def get_ip_address_from_interface(interface):
+            """Get device ip from target network interface."""
+            # android >= 6.0: ip -f inet addr show {interface}
+            try:
+                res = self.shell('ip -f inet addr show {}'.format(interface))
+            except AdbShellError:
+                res = ''
+            matcher = re.search(r"inet (\d+\.){3}\d+", res)
+            if matcher:
+                return matcher.group().split(" ")[-1]
+
+            # android >= 6.0 backup method: ifconfig
+            try:
+                res = self.shell('ifconfig')
+            except AdbShellError:
+                res = ''
+            matcher = re.search(interface + r'.*?inet addr:((\d+\.){3}\d+)', res, re.DOTALL)
+            if matcher:
+                return matcher.group(1)
+
+            # android <= 6.0: netcfg
             try:
                 res = self.shell('netcfg')
             except AdbShellError:
@@ -1270,22 +1303,17 @@ class ADB(object):
             matcher = re.search(interface + r'.* ((\d+\.){3}\d+)/\d+', res)
             if matcher:
                 return matcher.group(1)
-            else:
-                try:
-                    res = self.shell('ifconfig')
-                except AdbShellError:
-                    res = ''
-                matcher = re.search(interface + r'.*?inet addr:((\d+\.){3}\d+)', res, re.DOTALL)
-                if matcher:
-                    return matcher.group(1)
-                else:
-                    try:
-                        res = self.shell('getprop dhcp.{}.ipaddress'.format(interface))
-                    except AdbShellError:
-                        res = ''
-                    matcher = IP_PATTERN.search(res)
-                    if matcher:
-                        return matcher.group(0)
+
+            # android <= 6.0 backup method: getprop dhcp.{}.ipaddress
+            try:
+                res = self.shell('getprop dhcp.{}.ipaddress'.format(interface))
+            except AdbShellError:
+                res = ''
+            matcher = IP_PATTERN.search(res)
+            if matcher:
+                return matcher.group(0)
+
+            # sorry, no more methods...
             return None
 
         interfaces = ('eth0', 'eth1', 'wlan0')
@@ -1293,11 +1321,12 @@ class ADB(object):
             ip = get_ip_address_from_interface(i)
             if ip and not ip.startswith('172.') and not ip.startswith('127.') and not ip.startswith('169.'):
                 return ip
+
         return None
 
     def get_gateway_address(self):
         """
-        Perform several set of commands to obtain the gateway address
+        Perform several set of commands to obtain the gateway address.
             * `adb getprop dhcp.wlan0.gateway`
             * `adb shell netcfg | grep wlan0`
 
@@ -1459,6 +1488,55 @@ class ADB(object):
                 ret[k] = v
         return ret
 
+    def get_display_of_all_screen(self, info):
+        """
+        Perform `adb shell dumpsys window windows` commands to get window display of application.
+
+        Args:
+            info: device screen properties
+
+        Returns:
+            None if adb command failed to run, otherwise return device screen properties
+
+        """
+        output = self.shell("dumpsys window windows")
+        windows = output.split("Window #")
+        offsetx, offsety, x, y = info['width'], info['height'], 0, 0
+        package = self._search_for_current_package(output)
+        if package:
+            for w in windows:
+                if "package=%s" % package in w:
+                    arr = re.findall(r'Frames: containing=\[(\d+\.?\d*),(\d+\.?\d*)]\[(\d+\.?\d*),(\d+\.?\d*)]', w)
+                    if len(arr) >= 1 and len(arr[0]) == 4:
+                        offsetx, offsety, x, y = float(arr[0][0]), float(arr[0][1]), float(arr[0][2]), float(arr[0][3])
+                        if info["orientation"] in [1, 3]:
+                            offsetx, offsety, x, y = offsety, offsetx, y, x
+                        x, y = x - offsetx, y - offsety
+        return {
+            "offset_x": offsetx,
+            "offset_y": offsety,
+            "offset_width": x,
+            "offset_height": y
+        }
+
+    def _search_for_current_package(self, ret):
+        """
+        Search for current app package name from the output of command "adb shell dumpsys window windows"
+
+        Returns:
+            package name if exists else ""
+        """
+        try:
+            packageRE = re.compile('\s*mCurrentFocus=Window{.* ([A-Za-z0-9_.]+)/[A-Za-z0-9_.]+}')
+            m = packageRE.findall(ret)
+            if m:
+                return m[-1]
+            else:
+                return self.get_top_activity()[0]
+        except Exception as e:
+            print("[Error] Cannot get current top activity")
+        return ""
+
 
 def cleanup_adb_forward():
     for adb in ADB._instances:
