@@ -12,8 +12,11 @@ class Settings(object):
     CVSTRATEGY = ["surf", "tpl", "brisk"]  # keypoint matching: kaze/brisk/akaze/orb, contrib: sift/surf/brief
     KEYPOINT_MATCHING_PREDICTION = True
     THRESHOLD = 0.7  # [0, 1]
-    THRESHOLD_STRICT = 0.7  # [0, 1]
+    THRESHOLD_STRICT = None  # dedicated parameter for assert_exists
     OPDELAY = 0.1
     FIND_TIMEOUT = 20
     FIND_TIMEOUT_TMP = 3
     PROJECT_ROOT = os.environ.get("PROJECT_ROOT", "")  # for ``using`` other script
+    SNAPSHOT_QUALITY = 10  # 1-100 https://pillow.readthedocs.io/en/5.1.x/handbook/image-file-formats.html#jpeg
+    # Image compression size, e.g. 1200, means that the size of the screenshot does not exceed 1200*1200
+    IMAGE_MAXSIZE = os.environ.get("IMAGE_MAXSIZE", None)
