@@ -6,17 +6,22 @@ import io
 import re
 import six
 import sys
+from PIL import Image
 import shutil
 import jinja2
 import traceback
 from copy import deepcopy
+from datetime import datetime
 from jinja2 import evalcontextfilter, Markup, escape
 from airtest.aircv import imread, get_resolution
+from airtest.core.settings import Settings as ST
+from airtest.aircv.utils import compress_image
 from airtest.utils.compat import decode_path, script_dir_name
 from airtest.cli.info import get_script_info
+from airtest.utils.logger import get_logger
 from six import PY3
-from pprint import pprint
 
+LOGGING = get_logger(__name__)
 LOGDIR = "log"
 LOGFILE = "log.txt"
 HTML_TPL = "log_template.html"
@@ -26,6 +31,7 @@ STATIC_DIR = os.path.dirname(__file__)
 
 _paragraph_re = re.compile(r'(?:\r\n|\r|\n){2,}')
 
+
 @evalcontextfilter
 def nl2br(eval_ctx, value):
     result = u'\n\n'.join(u'<p>%s</p>' % p.replace('\n', '<br>\n')
@@ -34,6 +40,16 @@ def nl2br(eval_ctx, value):
         result = Markup(result)
     return result
 
+
+def timefmt(timestamp):
+    """
+    Formatting of timestamp in Jinja2 templates
+    :param timestamp: timestamp of steps
+    :return: "%Y-%m-%d %H:%M:%S"
+    """
+    return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")
+
+
 class LogToHtml(object):
     """Convert log to html display """
     scale = 0.5
@@ -57,11 +73,11 @@ class LogToHtml(object):
         if not plugins:
             return
         for plugin_name in plugins:
-            print("try loading plugin: %s" % plugin_name)
+            LOGGING.debug("try loading plugin: %s" % plugin_name)
             try:
                 __import__(plugin_name)
             except:
-                traceback.print_exc()
+                LOGGING.error(traceback.format_exc())
 
     def _load(self):
         logfile = self.logfile.encode(sys.getfilesystemencoding()) if not PY3 else self.logfile
@@ -78,7 +94,7 @@ class LogToHtml(object):
             depth = log['depth']
 
             if not self.run_start:
-                self.run_start = log["time"]
+                self.run_start = log.get('data', {}).get('start_time', '') or log["time"]
             self.run_end = log["time"]
 
             if depth == 0:
@@ -92,7 +108,6 @@ class LogToHtml(object):
             else:
                 children_steps.insert(0, log)
 
-        # pprint(steps)
         translated_steps = [self._translate_step(s) for s in steps]
         return translated_steps
 
@@ -103,11 +118,11 @@ class LogToHtml(object):
         code = self._translate_code(step)
         desc = self._translate_desc(step, code)
         screen = self._translate_screen(step, code)
-        traceback = self._translate_traceback(step)
+        info = self._translate_info(step)
         assertion = self._translate_assertion(step)
 
         # set test failed if any traceback exists
-        if traceback:
+        if info[0]:
             self.test_result = False
 
         translated = {
@@ -116,8 +131,9 @@ class LogToHtml(object):
             "code": code,
             "screen": screen,
             "desc": desc,
-            "traceback": traceback,
-            "assert": assertion
+            "traceback": info[0],
+            "log": info[1],
+            "assert": assertion,
         }
         return translated
 
@@ -126,7 +142,7 @@ class LogToHtml(object):
             return step["data"]["call_args"]["msg"]
 
     def _translate_screen(self, step, code):
-        if step['tag'] != "function":
+        if step['tag'] not in ["function", "info"] or not step.get("__children__"):
             return None
         screen = {
             "src": None,
@@ -137,14 +153,22 @@ class LogToHtml(object):
         }
 
         for item in step["__children__"]:
-            if item["data"]["name"] == "try_log_screen" and isinstance(item["data"].get("ret", None), six.text_type):
-                src = item["data"]['ret']
+            if item["data"]["name"] == "try_log_screen":
+                snapshot = item["data"].get("ret", None)
+                if isinstance(snapshot, six.text_type):
+                    src = snapshot
+                elif isinstance(snapshot, dict):
+                    src = snapshot['screen']
+                    screen['resolution'] = snapshot['resolution']
+                else:
+                    continue
                 if self.export_dir:  # all relative path
-                    src = os.path.join(LOGDIR, src)
-                    screen['_filepath'] = src
+                    screen['_filepath'] = os.path.join(LOGDIR, src)
                 else:
                     screen['_filepath'] = os.path.abspath(os.path.join(self.log_root, src))
                 screen['src'] = screen['_filepath']
+                self.get_thumbnail(os.path.join(self.log_root, src))
+                screen['thumbnail'] = self.get_small_name(screen['src'])
                 break
 
         display_pos = None
@@ -178,9 +202,35 @@ class LogToHtml(object):
             screen["pos"].append(display_pos)
         return screen
 
-    def _translate_traceback(self, step):
+    @classmethod
+    def get_thumbnail(cls, path):
+        """compress screenshot"""
+        new_path = cls.get_small_name(path)
+        if not os.path.isfile(new_path):
+            try:
+                img = Image.open(path)
+                compress_image(img, new_path, ST.SNAPSHOT_QUALITY)
+            except Exception:
+                LOGGING.error(traceback.format_exc())
+            return new_path
+        else:
+            return None
+
+    @classmethod
+    def get_small_name(cls, filename):
+        name, ext = os.path.splitext(filename)
+        return "%s_small%s" % (name, ext)
+
+    def _translate_info(self, step):
+        trace_msg, log_msg = "", ""
         if "traceback" in step["data"]:
-            return step["data"]["traceback"]
+            # 若包含有traceback内容，将会认定步骤失败
+            trace_msg = step["data"]["traceback"]
+        if step["tag"] == "info":
+            if "log" in step["data"]:
+                # 普通文本log内容，仅显示
+                log_msg = step["data"]["log"]
+        return trace_msg, log_msg
 
     def _translate_code(self, step):
         if step["tag"] != "function":
@@ -201,12 +251,16 @@ class LogToHtml(object):
             if isinstance(value, dict) and value.get("__class__") == "Template":
                 if self.export_dir:  # all relative path
                     image_path = value['filename']
-                    if not os.path.isfile(os.path.join(self.script_root, image_path)):
-                        shutil.copy(value['_filepath'], self.script_root)  # copy image used by using statement
+                    if not os.path.isfile(os.path.join(self.script_root, image_path)) and value['_filepath']:
+                        # copy image used by using statement
+                        shutil.copyfile(value['_filepath'], os.path.join(self.script_root, value['filename']))
                 else:
                     image_path = os.path.abspath(value['_filepath'] or value['filename'])
                 arg["image"] = image_path
-                crop_img = imread(value['_filepath'] or value['filename'])
+                if not value['_filepath'] and not os.path.exists(value['filename']):
+                    crop_img = imread(os.path.join(self.script_root, value['filename']))
+                else:
+                    crop_img = imread(value['_filepath'] or value['filename'])
                 arg["resolution"] = get_resolution(crop_img)
         return code
 
@@ -291,13 +345,14 @@ class LogToHtml(object):
             autoescape=True
         )
         env.filters['nl2br'] = nl2br
+        env.filters['datetime'] = timefmt
         template = env.get_template(template_name)
         html = template.render(**template_vars)
 
         if output_file:
             with io.open(output_file, 'w', encoding="utf-8") as f:
                 f.write(html)
-            print(output_file)
+            LOGGING.info(output_file)
 
         return html
 
@@ -307,18 +362,25 @@ class LogToHtml(object):
     def copy_tree(self, src, dst, ignore=None):
         try:
             shutil.copytree(src, dst, ignore=ignore)
-        except Exception as e:
-            print(e)
+        except:
+            LOGGING.error(traceback.format_exc())
 
     def _make_export_dir(self):
         """mkdir & copy /staticfiles/screenshots"""
-        dirname = os.path.basename(self.script_root).replace(".air", ".log")
+        # let dirname = <script name>.log
+        dirname = self.script_name.replace(os.path.splitext(self.script_name)[1], ".log")
         # mkdir
         dirpath = os.path.join(self.export_dir, dirname)
         if os.path.isdir(dirpath):
             shutil.rmtree(dirpath, ignore_errors=True)
+
         # copy script
-        self.copy_tree(self.script_root, dirpath)
+        def ignore_export_dir(dirname, filenames):
+            # 忽略当前导出的目录，防止递归导出
+            if os.path.commonprefix([dirpath, dirname]) == dirpath:
+                return filenames
+            return []
+        self.copy_tree(self.script_root, dirpath, ignore=ignore_export_dir)
         # copy log
         logpath = os.path.join(dirpath, LOGDIR)
         if os.path.normpath(logpath) != os.path.normpath(self.log_root):
@@ -332,21 +394,54 @@ class LogToHtml(object):
 
         return dirpath, logpath
 
-    def report(self, template_name, output_file=None, record_list=None):
+    def get_relative_log(self, output_file):
+        """
+        Try to get the relative path of log.txt
+        :param output_file: output file: log.html
+        :return: ./log.txt or ""
+        """
+        try:
+            html_dir = os.path.dirname(output_file)
+            return os.path.relpath(os.path.join(self.log_root, 'log.txt'), html_dir)
+        except:
+            LOGGING.error(traceback.format_exc())
+            return ""
+
+    def get_console(self, output_file):
+        html_dir = os.path.dirname(output_file)
+        file = os.path.join(html_dir, 'console.txt')
+        content = ""
+        if os.path.isfile(file):
+            try:
+                content = self.readFile(file)
+            except Exception:
+                try:
+                    content = self.readFile(file, "gbk")
+                except Exception:
+                    content = traceback.format_exc() + content
+                    content = content + "Can not read console.txt. Please check file in:\n" + file
+        return content
+
+    def readFile(self, filename, code='utf-8'):
+        content = ""
+        with io.open(filename, encoding=code) as f:
+            for line in f.readlines():
+                content = content + line
+        return content
+
+    def report_data(self, output_file=None, record_list=None):
+        """
+        Generate data for the report page
+        :param output_file: The file name or full path of the output file, default HTML_FILE
+        :param record_list: List of screen recording files
+        :return:
+        """
         self._load()
         steps = self._analyse()
 
         script_path = os.path.join(self.script_root, self.script_name)
         info = json.loads(get_script_info(script_path))
 
-        if self.export_dir:
-            self.script_root, self.log_root = self._make_export_dir()
-            output_file = os.path.join(self.script_root, HTML_FILE)
-            if not self.static_root.startswith("http"):
-                self.static_root = "static/"
-
-        if not record_list:
-            record_list = [f for f in os.listdir(self.log_root) if f.endswith(".mp4")]
         records = [os.path.join(LOGDIR, f) if self.export_dir
                    else os.path.abspath(os.path.join(self.log_root, f)) for f in record_list]
 
@@ -365,7 +460,35 @@ class LogToHtml(object):
         data['lang'] = self.lang
         data['records'] = records
         data['info'] = info
+        data['log'] = self.get_relative_log(output_file)
+        data['console'] = self.get_console(output_file)
+        # 如果带有<>符号，容易被highlight.js认为是特殊语法，有可能导致页面显示异常，尝试替换成不常用的{}
+        info = json.dumps(data).replace("<", "{").replace(">", "}")
+        data['data'] = info
+        return data
 
+    def report(self, template_name=HTML_TPL, output_file=None, record_list=None):
+        """
+        Generate the report page, you can add custom data and overload it if needed
+        :param template_name: default is HTML_TPL
+        :param output_file: The file name or full path of the output file, default HTML_FILE
+        :param record_list: List of screen recording files
+        :return:
+        """
+        if not self.script_name:
+            path, self.script_name = script_dir_name(self.script_root)
+
+        if self.export_dir:
+            self.script_root, self.log_root = self._make_export_dir()
+            # output_file可传入文件名，或绝对路径
+            output_file = output_file if output_file and os.path.isabs(output_file) \
+                else os.path.join(self.script_root, output_file or HTML_FILE)
+            if not self.static_root.startswith("http"):
+                self.static_root = "static/"
+
+        if not record_list:
+            record_list = [f for f in os.listdir(self.log_root) if f.endswith(".mp4")]
+        data = self.report_data(output_file=output_file, record_list=record_list)
         return self._render(template_name, output_file, **data)
 
 
